@@ -111,12 +111,13 @@ class DistUpgradeQuirks(object):
         logging.debug("running Quirks.PreCacheOpen")
 
     # individual quirks handler that run *after* the cache is opened
-    def groovyPostInitialUpdate(self):
+    def impishPostInitialUpdate(self):
         # PreCacheOpen would be better but controller.abort fails terribly
         """ run after the apt cache is opened the first time """
-        logging.debug("running Quirks.groovyPostInitialUpdate")
+        logging.debug("running Quirks.impishPostInitialUpdate")
         self._get_from_and_to_version()
         self._test_and_fail_on_i386()
+        self._test_and_fail_on_aufs()
 
         cache = self.controller.cache
         self._test_and_warn_if_ros_installed(cache)
@@ -132,8 +133,8 @@ class DistUpgradeQuirks(object):
         if self._snapstore_reachable:
             self._calculateSnapSizeRequirements()
 
-    def groovyPostUpgrade(self):
-        logging.debug("running Quirks.groovyPostUpgrade")
+    def impishPostUpgrade(self):
+        logging.debug("running Quirks.impishPostUpgrade")
         cache = self.controller.cache
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
@@ -141,6 +142,12 @@ class DistUpgradeQuirks(object):
         if cache['snapd'].is_installed and \
                 self._snap_list:
             self._replaceDebsAndSnaps()
+        if 'ubuntu-desktop-raspi' in cache:
+            if cache['ubuntu-desktop-raspi'].is_installed:
+                self._replace_fkms_overlay()
+        if 'linux-firmware-raspi2' in cache:
+            if cache['linux-firmware-raspi2'].is_installed:
+                self._remove_uboot_on_rpi()
 
     # individual quirks handler when the dpkg run is finished ---------
     def PostCleanup(self):
@@ -163,7 +170,7 @@ class DistUpgradeQuirks(object):
     def PreDistUpgradeCache(self):
         """ run right before calculating the dist-upgrade """
         logging.debug("running Quirks.PreDistUpgradeCache")
-        self._install_python_is_python2()
+        # self._install_python_is_python2()
 
     # individual quirks handler that run *after* the dist-upgrade was
     # calculated in the cache
@@ -340,6 +347,28 @@ class DistUpgradeQuirks(object):
                       "with this hardware."))
                 self.controller.abort()
 
+    def _test_and_fail_on_aufs(self):
+        """
+        Test and fail if docker has aufs storage driver available
+        as it is deprecated after 20.04. Even if no containers are
+        currently running aufs, the upgrade could break existing
+        container images.
+        """
+        for aufs_dir in ("/var/snap/docker/common/var-lib-docker/aufs",
+                         "/var/lib/docker/aufs"):
+            if (os.path.exists(aufs_dir)):
+                logging.error("Docker config uses aufs")
+                summary = _("Sorry, this storage driver is not supported "
+                            "in kernels for newer releases")
+                msg = _("There will not be any further Ubuntu releases "
+                        "that provide kernel support for the aufs "
+                        "storage driver.\n\n"
+                        "Please ensure that none of your containers are "
+                        "using the aufs storage driver, remove the directory "
+                        "%s and try again." % aufs_dir)
+                self._view.error(summary, msg)
+                self.controller.abort()
+
     def _test_and_warn_if_vserver(self):
         """
         upstart and vserver environments are not a good match, warn
@@ -424,6 +453,19 @@ class DistUpgradeQuirks(object):
             return False
         return True
 
+    def _readDMIVendor(self):
+        """
+        read /sys/class/dmi/id/sys_vendor in case there are
+        system specific issues with upgrading e.g LP: #1928434
+        """
+        logging.debug("checking for system vendor")
+        if not os.path.exists("/sys/class/dmi/id/sys_vendor"):
+            logging.error("cannot open /sys/class/dmi/id/sys_vendor")
+            return ''
+        with open("/sys/class/dmi/id/sys_vendor") as f:
+            vendor = f.read()
+        return vendor
+
     def _stopApparmor(self):
         """ /etc/init.d/apparmor stop (see bug #559433)"""
         if os.path.exists("/etc/init.d/apparmor"):
@@ -548,6 +590,8 @@ class DistUpgradeQuirks(object):
         elif 'error: access denied' in connected[1]:
             res = False
             logging.error("Not running as root!")
+        else:
+            logging.error("Unhandled error connecting to the snap store.")
         if not res:
             self.controller.abort()
 
@@ -812,7 +856,7 @@ class DistUpgradeQuirks(object):
         pattern = re.compile('(.+)-([0-9]+)-(.+)')
         match1 = pattern.match(term1)
         match2 = pattern.match(term2)
-        if match1:
+        if match1 and match2:
             term1 = '%s-%s' % (match1.group(1),
                                match1.group(2))
             term2 = '%s-%s' % (match2.group(1),
@@ -903,27 +947,64 @@ class DistUpgradeQuirks(object):
                         ('libpython-stdlib', None),
                         ('libpython-dbg', None))
         cache = self.controller.cache
+
+        # We run with auto_inst=False first to mark our explicit choices so it
+        # can guide the code with auto_inst=True when it makes decisions
+        for auto_inst in False, True:
+            for old, new in replacements:
+                logging.info("checking for %s (auto_inst=%s)" %
+                             (old, auto_inst))
+                if old in cache and cache[old].is_installed:
+                    if new:
+                        logging.info("installing %s because %s was installed" %
+                                     (new, old))
+                        reason = "%s was installed on the system" % old
+                        if not cache.mark_install(new, reason, auto_fix=False,
+                                                  auto_inst=auto_inst):
+                            logging.info("failed to install %s" % new)
+                    logging.info("removing %s because %s is being installed" %
+                                 (old, new))
+                    reason = "%s is being installed on the system" % new
+                    if not cache.mark_remove(old, reason, auto_fix=False):
+                        logging.info("failed to remove %s", old)
+
+        solver = apt.cache.ProblemResolver(cache)
+
         for old, new in replacements:
-            logging.info("checking for %s" % old)
             if old in cache and cache[old].is_installed:
                 if new:
-                    logging.info("installing %s because %s was installed" %
-                                 (new, old))
-                    reason = "%s was installed on the system" % old
-                    if not cache.mark_install(new, reason):
-                        logging.info("failed to install %s" % new)
-                logging.info("removing %s because %s is being installed" %
-                             (old, new))
-                reason = "%s is being installed on the system" % new
-                if not cache.mark_remove(old, reason):
-                    logging.info("failed to remove %s", old)
+                    if new not in cache:
+                        old_inst = [old for old, new in replacements if old
+                                    in cache and cache[old].is_installed]
+                        summary = _("universe component not enabled")
+                        msg = _("You have the package %s installed which "
+                                "is a python2 package. python2 has been "
+                                "deprecated in Ubuntu 20.04 LTS and is now "
+                                "available from the universe component of "
+                                "the Ubuntu archive. To ensure any existing "
+                                "third party code you have installed "
+                                "continues to work the release upgrade "
+                                "process would install %s for you. However, "
+                                "the universe component of the archive is not "
+                                "enabled on this system. For the upgrade "
+                                "process to proceed you should either enable "
+                                "the universe component in "
+                                "/etc/apt/sources.list or remove %s"
+                                % (old, new, ', '.join(old_inst))
+                                )
+                        logging.info(summary)
+                        # necessary to make the error text available again on
+                        # stdout for the text frontend
+                        cache._stopAptResolverLog()
+                        self._view.error(summary, msg)
+                        self.controller.abort()
+                    else:
+                        solver.clear(cache[new])
+                        solver.protect(cache[new])
+                solver.clear(cache[old])
+                solver.remove(cache[old])
 
-            # protect our decision to remove legacy 'python' (as a
-            # dependency of python-minimal, removed above)
-            py = 'python'
-            if py in cache and cache[py].marked_delete:
-                resolver = apt.cache.ProblemResolver(cache)
-                resolver.protect(cache[py])
+        solver.resolve()
 
     def ensure_recommends_are_installed_on_desktops(self):
         """ ensure that on a desktop install recommends are installed
@@ -1076,3 +1157,187 @@ class DistUpgradeQuirks(object):
 
                 self._snap_list[snap] = snap_object
         return self._snap_list
+
+    def _replace_fkms_overlay(self, boot_dir='/boot/firmware'):
+        failure_action = (
+            "You may need to replace the vc4-fkms-v3d overlay with "
+            "vc4-kms-v3d in config.txt on your boot partition")
+
+        try:
+            boot_config_filename = os.path.join(boot_dir, 'config.txt')
+            with open(boot_config_filename, 'r', encoding='utf-8') as f:
+                boot_config = f.read()
+        except FileNotFoundError:
+            logging.error("failed to open boot configuration in %s; %s",
+                          boot_config_filename, failure_action)
+            return
+
+        new_config = ''.join(
+            # startswith and replace used to cope with (and preserve) any
+            # trailing d-t parameters, and any use of the -pi4 suffix
+            '# changed by do-release-upgrade (LP: #1923673)\n#' + line +
+            line.replace('dtoverlay=vc4-fkms-v3d', 'dtoverlay=vc4-kms-v3d')
+            if line.startswith('dtoverlay=vc4-fkms-v3d') else
+            # camera firmware disabled due to incompatibility with "full" kms
+            # overlay; without the camera firmware active it's also better to
+            # disable gpu_mem leaving the default (64MB) to allow as much as
+            # possible for the KMS driver
+            '# disabled by do-release-upgrade (LP: #1923673)\n#' + line
+            if line.startswith('gpu_mem=') or line.rstrip() == 'start_x=1' else
+            line
+            for line in boot_config.splitlines(keepends=True)
+        )
+
+        if new_config == boot_config:
+            logging.warning("no fkms overlay or camera firmware line found "
+                            "in %s", boot_config_filename)
+            return
+
+        try:
+            boot_backup_filename = os.path.join(
+                boot_dir, 'config.txt.distUpgrade')
+            with open(boot_backup_filename, 'w', encoding='utf-8') as f:
+                f.write(boot_config)
+        except IOError as exc:
+            logging.error("unable to write boot config backup to %s: %s; %s",
+                          boot_backup_filename, exc, failure_action)
+            return
+        try:
+            with open(boot_config_filename, 'w', encoding='utf-8') as f:
+                f.write(new_config)
+        except IOError as exc:
+            logging.error("unable to write new boot config to %s: %s; %s",
+                          boot_config_filename, exc, failure_action)
+
+    def _remove_uboot_on_rpi(self, boot_dir='/boot/firmware'):
+        kernel_line = 'kernel=vmlinuz'
+        initramfs_line = 'initramfs initrd.img followkernel' 
+        failure_action = (
+            "You may need to replace u_boot_* with vmlinuz, and add "
+            "{initramfs_line!r} to config.txt on your boot "
+            "partition; see LP: #1936401 for further details")
+        failure_action = failure_action.format(initramfs_line=initramfs_line)
+        change_prefix = '# commented by do-release-upgrade (LP: #1936401)'
+        added_prefix = '# added by do-release-upgrade (LP: #1936401)'
+        merge_prefix = '# merged from {} by do-release-upgrade (LP: #1936401)'
+
+        try:
+            boot_config_filename = os.path.join(boot_dir, 'config.txt')
+            with open(boot_config_filename, 'r', encoding='utf-8') as f:
+                boot_config = f.read()
+        except FileNotFoundError:
+            logging.error("failed to open boot configuration in %s; %s",
+                          boot_config_filename, failure_action)
+            return
+
+        def replace_uboot(lines):
+            result = []
+            removed_uboot = added_kernel = False
+            for line in lines:
+                if line == '[all]':
+                    # Add the vmlinuz kernel and initrd.img to the first
+                    # encountered [all] section, just in case the user has any
+                    # custom kernel overrides later in the sequence
+                    result.append(line)
+                    if not added_kernel:
+                        result.append(added_prefix)
+                        result.append(kernel_line)
+                        result.append(initramfs_line)
+                        added_kernel = True
+                elif line.startswith('device_tree_address='):
+                    # Disable any device_tree_address= line (leave the boot
+                    # firmware to place this rather than risk trampling over it
+                    # with kernel/initrd)
+                    result.append(change_prefix)
+                    result.append('#' + line)
+                elif line.startswith('kernel=uboot_rpi_'):
+                    # Disable all kernel=uboot_rpi_* lines found in the
+                    # configuration
+                    removed_uboot = True
+                    result.append(change_prefix)
+                    result.append('#' + line)
+                else:
+                    result.append(line)
+            # We don't *want* to touch the config unless we absolutely have to,
+            # and the user may have already performed this conversion (e.g. to
+            # take advantage of USB MSD boot), or otherwise customized their
+            # boot configuration. Hence, if we didn't find (and remove) any
+            # u-boot entries, assume the kernel/initrd config was already fine
+            # and return the verbatim config
+            if removed_uboot:
+                if not added_kernel:
+                    result.append(added_prefix)
+                    result.append('[all]')
+                    result.append(kernel_line)
+                    result.append(initramfs_line)
+                return result
+            else:
+                return lines
+
+        def merge_includes(lines):
+            result = []
+            skip_comments = True
+            found_includes = False
+            for line in lines:
+                # Skip the initial comment block warning that config.txt is not
+                # to be edited (the usercfg.txt and syscfg.txt files were
+                # merged in the groovy release, but not for upgraders)
+                if line.startswith('#') and skip_comments:
+                    continue
+                skip_comments = False
+                if line in ('include syscfg.txt', 'include usercfg.txt'):
+                    # Merge the usercfg.txt and syscfg.txt configuration files
+                    # into config.txt, skipping the initial comments. Note that
+                    # we don't bother with other includes the user has added
+                    # themselves (as they presumably want to keep those)
+                    found_includes = True
+                    included_filename = line.split(maxsplit=1)[1]
+                    result.append(merge_prefix.format(included_filename))
+                    included_filename = os.path.join(boot_dir,
+                                                     included_filename)
+                    skip_comments = True
+                    with open(included_filename, 'r', encoding='utf-8') as f:
+                        for line in f:
+                            if line.startswith('#') and skip_comments:
+                                continue
+                            skip_comments = False
+                            result.append(line.rstrip())
+                    target_filename = included_filename + '.distUpgrade'
+                    try:
+                        os.rename(included_filename, target_filename)
+                    except IOError as exc:
+                        logging.error("failed to move included configuration "
+                                      "from %s to %s; %s", included_filename,
+                                      target_filename, exc)
+                else:
+                    result.append(line)
+            # Again, if we found no includes, return the original verbatim
+            # (i.e. with initial comments intact)
+            if found_includes:
+                return result
+            else:
+                return lines
+
+        lines = [line.rstrip() for line in boot_config.splitlines()]
+        lines = replace_uboot(merge_includes(lines))
+        new_config = ''.join(line + '\n' for line in lines)
+
+        if new_config == boot_config:
+            logging.warning("no u-boot removal performed in %s",
+                            boot_config_filename)
+            return
+
+        try:
+            boot_backup_filename = boot_config_filename + '.distUpgrade'
+            with open(boot_backup_filename, 'w', encoding='utf-8') as f:
+                f.write(boot_config)
+        except IOError as exc:
+            logging.error("unable to write boot config backup to %s: %s; %s",
+                          boot_backup_filename, exc, failure_action)
+            return
+        try:
+            with open(boot_config_filename, 'w', encoding='utf-8') as f:
+                f.write(new_config)
+        except IOError as exc:
+            logging.error("unable to write new boot config to %s: %s; %s",
+                          boot_config_filename, exc, failure_action)
