@@ -9,8 +9,9 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import os, glob, subprocess, os.path, time, pwd, sys, requests_unixsocket
-import stat
+import os, glob, subprocess, os.path, time, pwd, sys, stat, json, socket
+import http.client
+from contextlib import closing
 from operator import itemgetter
 
 try:
@@ -32,6 +33,23 @@ max_corefiles_per_uid = 5
 
 _config_file = '~/.config/apport/settings'
 
+SNAPD_SOCKET = '/run/snapd.socket'
+
+
+#  UHTTPConnection is based on code from the UpdateManager package:
+#  Copyright (c) 2017 Canonical
+#  Author: Andrea Azzarone <andrea.azzarone@canonical.com>
+class UHTTPConnection(http.client.HTTPConnection, object):
+
+    def __init__(self, path):
+        http.client.HTTPConnection.__init__(self, 'localhost')
+        self.path = path
+
+    def connect(self):
+        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        sock.connect(self.path)
+        self.sock = sock
+
 
 def allowed_to_report():
     '''Check whether crash reporting is enabled.'''
@@ -45,6 +63,33 @@ def allowed_to_report():
         return False
 
 
+def get_dbus_socket(dbus_addr):
+    '''Extract the socket from a DBus address.'''
+
+    if not dbus_addr:
+        return None
+
+    # Only support unix domain sockets, and only the default Ubuntu path
+    if not dbus_addr.startswith("unix:path=/run/user/"):
+        return None
+
+    # Prevent path traversal
+    if "../" in dbus_addr:
+        return None
+
+    # Don't support escaped values, multiple addresses, or multiple keys
+    # and values
+    for search in ["%", ",", ";"]:
+        if search in dbus_addr:
+            return None
+
+    parts = dbus_addr.split("=")
+    if len(parts) != 2:
+        return None
+
+    return parts[1]
+
+
 def find_package_desktopfile(package):
     '''Return a package's .desktop file.
 
@@ -115,12 +160,13 @@ def find_snap(snap):
 
     Return None if the snap is not found to be installed.
     '''
-    session = requests_unixsocket.Session()
     try:
-        r = session.get('http+unix://%2Frun%2Fsnapd.socket/v2/snaps/{}'.format(snap))
-        if r.status_code == 200:
-            j = r.json()
-            return j["result"]
+        with closing(UHTTPConnection(SNAPD_SOCKET)) as c:
+            url = f'/v2/snaps/{snap}'
+            c.request('GET', url)
+            response = c.getresponse()
+            if response.status == 200:
+                return json.loads(response.read())['result']
     except Exception:
         return None
 
@@ -360,7 +406,7 @@ def get_config(section, setting, default=None, path=None, bool=False):
     fd = None
     f = None
     if not get_config.config:
-        get_config.config = ConfigParser()
+        get_config.config = ConfigParser(interpolation=None)
 
         try:
             fd = os.open(path, os.O_NOFOLLOW | os.O_RDONLY)
@@ -422,6 +468,22 @@ def get_uid_and_gid(contents):
     return (real_uid, real_gid)
 
 
+def search_map(mapfd, uid):
+    '''Search for an ID in a map fd'''
+    for line in mapfd:
+        fields = line.split()
+        if len(fields) != 3:
+            continue
+
+        host_start = int(fields[1])
+        host_end = host_start + int(fields[2])
+
+        if uid >= host_start and uid <= host_end:
+            return True
+
+    return False
+
+
 def get_boot_id():
     '''Gets the kernel boot id'''
 
@@ -430,7 +492,18 @@ def get_boot_id():
     return boot_id
 
 
-def get_core_path(pid=None, exe=None, uid=None, timestamp=None):
+def get_process_path(proc_pid_fd=None):
+    '''Gets the process path from a proc directory file descriptor'''
+
+    if proc_pid_fd is None:
+        return 'unknown'
+    try:
+        return os.readlink('exe', dir_fd=proc_pid_fd)
+    except OSError:
+        return 'unknown'
+
+
+def get_core_path(pid=None, exe=None, uid=None, timestamp=None, proc_pid_fd=None):
     '''Get the path to a core file'''
 
     if pid is None:
@@ -443,9 +516,8 @@ def get_core_path(pid=None, exe=None, uid=None, timestamp=None):
             timestamp = get_starttime(stat_contents)
 
     if exe is None:
-        exe = 'unknown'
-    else:
-        exe = exe.replace('/', '_').replace('.', '_')
+        exe = get_process_path(proc_pid_fd)
+    exe = exe.replace('/', '_').replace('.', '_')
 
     if uid is None:
         uid = os.getuid()
@@ -464,6 +536,7 @@ def find_core_files_by_uid(uid):
     '''Searches the core file directory for files that belong to a
        specified uid. Returns a list of lists containing the filename and
        the file modification time.'''
+    uid = str(uid)
     core_files = []
     uid_files = []
 
@@ -472,7 +545,7 @@ def find_core_files_by_uid(uid):
 
     for f in core_files:
         try:
-            if f.split('.')[2] == str(uid):
+            if f.split('.')[2] == uid:
                 time = os.path.getmtime(os.path.join(core_dir, f))
                 uid_files.append([f, time])
         except (IndexError, FileNotFoundError):
@@ -487,7 +560,7 @@ def clean_core_directory(uid):
     uid_files = find_core_files_by_uid(uid)
     sorted_files = sorted(uid_files, key=itemgetter(1))
 
-    # Substract a extra one to make room for the new core file
+    # Subtract a extra one to make room for the new core file
     if len(uid_files) > max_corefiles_per_uid - 1:
         for x in range(len(uid_files) - max_corefiles_per_uid + 1):
             os.remove(os.path.join(core_dir, sorted_files[0][0]))
