@@ -4,21 +4,22 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
-__VERSION__ = "21.2"
-_PACKAGED_VERSION = '21.2-3-g899bfaa9-0ubuntu2~20.10.1'
+__VERSION__ = "22.2"
+_PACKAGED_VERSION = "22.2-0ubuntu1~21.10.3"
 
 FEATURES = [
     # supports network config version 1
-    'NETWORK_CONFIG_V1',
+    "NETWORK_CONFIG_V1",
     # supports network config version 2 (netplan)
-    'NETWORK_CONFIG_V2',
+    "NETWORK_CONFIG_V2",
 ]
 
 
 def version_string():
     """Extract a version string from cloud-init."""
-    if not _PACKAGED_VERSION.startswith('@@'):
+    if not _PACKAGED_VERSION.startswith("@@"):
         return _PACKAGED_VERSION
     return __VERSION__
 
+
 # vi: ts=4 expandtab
