@@ -56,7 +56,7 @@ except ImportError:
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '1.22'
+__version__ = '1.25'
 __website__ = 'https://github.com/stefankoegl/python-json-patch'
 __license__ = 'Modified BSD License'
 
@@ -474,7 +474,7 @@ class ReplaceOperation(PatchOperation):
             return value
 
         if isinstance(subobj, MutableSequence):
-            if part > len(subobj) or part < 0:
+            if part >= len(subobj) or part < 0:
                 raise JsonPatchConflict("can't replace outside of list")
 
         elif isinstance(subobj, MutableMapping):
@@ -717,7 +717,7 @@ class DiffBuilder(object):
         index = self.take_index(item, _ST_REMOVE)
         if index is not None:
             op = index[2]
-            if type(op.key) == int:
+            if type(op.key) == int and type(key) == int:
                 for v in self.iter_from(index):
                     op.key = v._on_undo_remove(op.path, op.key)
 
@@ -817,10 +817,7 @@ class DiffBuilder(object):
                 self._item_added(path, key, dst[key])
 
     def _compare_values(self, path, key, src, dst):
-        if src == dst:
-            return
-
-        elif isinstance(src, MutableMapping) and \
+        if isinstance(src, MutableMapping) and \
                 isinstance(dst, MutableMapping):
             self._compare_dicts(_path_join(path, key), src, dst)
 
@@ -828,6 +825,16 @@ class DiffBuilder(object):
                 isinstance(dst, MutableSequence):
             self._compare_lists(_path_join(path, key), src, dst)
 
+        # To ensure we catch changes to JSON, we can't rely on a simple
+        # src == dst, or it would not recognize the difference between
+        # 1 and True, among other things. Using json.dumps is the most
+        # fool-proof way to ensure we catch type changes that matter to JSON
+        # and ignore those that don't. The performance of this could be
+        # improved by doing more direct type checks, but we'd need to be
+        # careful to accept type changes that don't matter when JSONified.
+        elif json.dumps(src) == json.dumps(dst):
+            return
+
         else:
             self._item_replaced(path, key, dst)
 
