@@ -17,7 +17,7 @@ import logging
 from datetime import datetime
 import glob
 import sos.report.plugins
-from sos.utilities import (ImporterHelper, SoSTimeoutError,
+from sos.utilities import (ImporterHelper, SoSTimeoutError, bold,
                            sos_get_command_output, TIMEOUT_DEFAULT)
 from shutil import rmtree
 import hashlib
@@ -82,6 +82,7 @@ class SoSReport(SoSComponent):
         'case_id': '',
         'chroot': 'auto',
         'clean': False,
+        'container_runtime': 'auto',
         'keep_binary_files': False,
         'desc': '',
         'domains': [],
@@ -173,14 +174,12 @@ class SoSReport(SoSComponent):
         self._set_directories()
 
         msg = "default"
-        host_sysroot = self.policy.host_sysroot()
+        self.sysroot = self.policy.sysroot
         # set alternate system root directory
         if self.opts.sysroot:
             msg = "cmdline"
-            self.sysroot = self.opts.sysroot
-        elif self.policy.in_container() and host_sysroot != os.sep:
+        elif self.policy.in_container() and self.sysroot != os.sep:
             msg = "policy"
-            self.sysroot = host_sysroot
         self.soslog.debug("set sysroot to '%s' (%s)" % (self.sysroot, msg))
 
         if self.opts.chroot not in chroot_modes:
@@ -189,6 +188,7 @@ class SoSReport(SoSComponent):
             self.tempfile_util.clean()
             self._exit(1)
 
+        self._check_container_runtime()
         self._get_hardware_devices()
         self._get_namespaces()
 
@@ -220,6 +220,9 @@ class SoSReport(SoSComponent):
                                 dest="chroot", default='auto',
                                 help="chroot executed commands to SYSROOT "
                                      "[auto, always, never] (default=auto)")
+        report_grp.add_argument("--container-runtime", default="auto",
+                                help="Default container runtime to use for "
+                                     "collections. 'auto' for policy control.")
         report_grp.add_argument("--desc", "--description", type=str,
                                 action="store", default="",
                                 help="Description for a new preset",)
@@ -339,7 +342,7 @@ class SoSReport(SoSComponent):
         cleaner_grp.add_argument('--clean', '--cleaner', '--mask',
                                  dest='clean',
                                  default=False, action='store_true',
-                                 help='Obfuscate sensistive information')
+                                 help='Obfuscate sensitive information')
         cleaner_grp.add_argument('--domains', dest='domains', default=[],
                                  action='extend',
                                  help='Additional domain names to obfuscate')
@@ -365,6 +368,62 @@ class SoSReport(SoSComponent):
                                  action='extend',
                                  help='List of usernames to obfuscate')
 
+    @classmethod
+    def display_help(cls, section):
+        section.set_title('SoS Report Detailed Help')
+        section.add_text(
+            'The report command is the most common use case for SoS, and aims '
+            'to collect relevant diagnostic and troubleshooting data to assist'
+            ' with issue analysis without actively performing that analysis on'
+            ' the system while it is in use.'
+        )
+        section.add_text(
+            'Additionally, sos report archives can be used for ongoing '
+            'inspection for pre-emptive issue monitoring, such as that done '
+            'by the Insights project.'
+        )
+
+        section.add_text(
+            'The typical result of an execution of \'sos report\' is a tarball'
+            ' that contains troubleshooting command output, copies of config '
+            'files, and copies of relevant sections of the host filesystem. '
+            'Root privileges are required for collections.'
+        )
+
+        psec = section.add_section(title='How Collections Are Determined')
+        psec.add_text(
+            'SoS report performs it\'s collections by way of \'plugins\' that '
+            'individually specify what files to copy and what commands to run.'
+            ' Plugins typically map to specific components or software '
+            'packages.'
+        )
+        psec.add_text(
+            'Plugins may specify different collections on different distribu'
+            'tions, and some plugins may only be for specific distributions. '
+            'Distributions are represented within SoS by \'policies\' and may '
+            'influence how other SoS commands or options function. For example'
+            'policies can alter where the --upload option defaults to or '
+            'functions.'
+        )
+
+        ssec = section.add_section(title='See Also')
+        ssec.add_text(
+            "For information on available options for report, see %s and %s"
+            % (bold('sos report --help'), bold('man sos-report'))
+        )
+        ssec.add_text("The following %s sections may be of interest:\n"
+                      % bold('sos help'))
+        help_lines = {
+            'report.plugins': 'Information on the plugin design of sos',
+            'report.plugins.$plugin': 'Information on a specific $plugin',
+            'policies': 'How sos operates on different distributions'
+        }
+        helpln = ''
+        for ln in help_lines:
+            ssec.add_text("\t{:<36}{}".format(ln, help_lines[ln]),
+                          newline=False)
+        ssec.add_text(helpln)
+
     def print_header(self):
         print("\n%s\n" % _("sosreport (version %s)" % (__version__,)))
 
@@ -375,6 +434,37 @@ class SoSReport(SoSComponent):
         }
         # TODO: enumerate network devices, preferably with devtype info
 
+    def _check_container_runtime(self):
+        """Check the loaded container runtimes, and the policy default runtime
+        (if set), against any requested --container-runtime value. This can be
+        useful for systems that have multiple runtimes, such as RHCOS, but do
+        not have a clearly defined 'default' (or one that is determined based
+        entirely on configuration).
+        """
+        if self.opts.container_runtime != 'auto':
+            crun = self.opts.container_runtime.lower()
+            if crun in ['none', 'off', 'diabled']:
+                self.policy.runtimes = {}
+                self.soslog.info(
+                    "Disabled all container runtimes per user option."
+                )
+            elif not self.policy.runtimes:
+                msg = ("WARNING: No container runtimes are active, ignoring "
+                       "option to set default runtime to '%s'\n" % crun)
+                self.soslog.warn(msg)
+            elif crun not in self.policy.runtimes.keys():
+                valid = ', '.join(p for p in self.policy.runtimes.keys()
+                                  if p != 'default')
+                raise Exception("Cannot use container runtime '%s': no such "
+                                "runtime detected. Available runtimes: %s"
+                                % (crun, valid))
+            else:
+                self.policy.runtimes['default'] = self.policy.runtimes[crun]
+                self.soslog.info(
+                    "Set default container runtime to '%s'"
+                    % self.policy.runtimes['default'].name
+                )
+
     def get_fibre_devs(self):
         """Enumerate a list of fibrechannel devices on this system so that
         plugins can iterate over them
@@ -626,21 +716,24 @@ class SoSReport(SoSComponent):
     def _set_all_options(self):
         if self.opts.alloptions:
             for plugname, plug in self.loaded_plugins:
-                for name, parms in zip(plug.opt_names, plug.opt_parms):
-                    if type(parms["enabled"]) == bool:
-                        parms["enabled"] = True
+                for opt in plug.options.values():
+                    if bool in opt.val_type:
+                        opt.value = True
 
     def _set_tunables(self):
         if self.opts.plugopts:
             opts = {}
             for opt in self.opts.plugopts:
-                # split up "general.syslogsize=5"
                 try:
                     opt, val = opt.split("=")
                 except ValueError:
                     val = True
-                else:
-                    if val.lower() in ["off", "disable", "disabled", "false"]:
+
+                if isinstance(val, str):
+                    val = val.lower()
+                    if val in ["on", "enable", "enabled", "true", "yes"]:
+                        val = True
+                    elif val in ["off", "disable", "disabled", "false", "no"]:
                         val = False
                     else:
                         # try to convert string "val" to int()
@@ -649,7 +742,6 @@ class SoSReport(SoSComponent):
                         except ValueError:
                             pass
 
-                # split up "general.syslogsize"
                 try:
                     plug, opt = opt.split(".")
                 except ValueError:
@@ -659,16 +751,25 @@ class SoSReport(SoSComponent):
                 try:
                     opts[plug]
                 except KeyError:
-                    opts[plug] = []
-                opts[plug].append((opt, val))
+                    opts[plug] = {}
+                opts[plug][opt] = val
 
             for plugname, plug in self.loaded_plugins:
                 if plugname in opts:
-                    for opt, val in opts[plugname]:
-                        if not plug.set_option(opt, val):
+                    for opt in opts[plugname]:
+                        if opt not in plug.options:
                             self.soslog.error('no such option "%s" for plugin '
                                               '(%s)' % (opt, plugname))
                             self._exit(1)
+                        try:
+                            plug.options[opt].set_value(opts[plugname][opt])
+                            self.soslog.debug(
+                                "Set %s plugin option to %s"
+                                % (plugname, plug.options[opt])
+                            )
+                        except Exception as err:
+                            self.soslog.error(err)
+                            self._exit(1)
                     del opts[plugname]
             for plugname in opts.keys():
                 self.soslog.error('WARNING: unable to set option for disabled '
@@ -695,10 +796,8 @@ class SoSReport(SoSComponent):
 
     def _set_plugin_options(self):
         for plugin_name, plugin in self.loaded_plugins:
-            names, parms = plugin.get_all_options()
-            for optname, optparm in zip(names, parms):
-                self.all_options.append((plugin, plugin_name, optname,
-                                         optparm))
+            for opt in plugin.options:
+                self.all_options.append(plugin.options[opt])
 
     def _set_estimate_only(self):
         # set estimate-only mode by enforcing some options settings
@@ -766,31 +865,33 @@ class SoSReport(SoSComponent):
         if self.all_options:
             self.ui_log.info(_("The following options are available for ALL "
                                "plugins:"))
-            for opt in self.all_options[0][0]._default_plug_opts:
-                val = opt[3]
-                if val == -1:
+            _defaults = self.loaded_plugins[0][1].get_default_plugin_opts()
+            for _opt in _defaults:
+                opt = _defaults[_opt]
+                val = opt.default
+                if opt.default == -1:
                     val = TIMEOUT_DEFAULT
-                self.ui_log.info(" %-25s %-15s %s" % (opt[0], val, opt[1]))
+                self.ui_log.info(" %-25s %-15s %s" % (opt.name, val, opt.desc))
             self.ui_log.info("")
 
             self.ui_log.info(_("The following plugin options are available:"))
-            for (plug, plugname, optname, optparm) in self.all_options:
-                if optname in ('timeout', 'postproc', 'cmd-timeout'):
+            for opt in self.all_options:
+                if opt.name in ('timeout', 'postproc', 'cmd-timeout'):
                     continue
                 # format option value based on its type (int or bool)
-                if type(optparm["enabled"]) == bool:
-                    if optparm["enabled"] is True:
+                if isinstance(opt.default, bool):
+                    if opt.default is True:
                         tmpopt = "on"
                     else:
                         tmpopt = "off"
                 else:
-                    tmpopt = optparm["enabled"]
+                    tmpopt = opt.default
 
                 if tmpopt is None:
                     tmpopt = 0
 
                 self.ui_log.info(" %-25s %-15s %s" % (
-                    plugname + "." + optname, tmpopt, optparm["desc"]))
+                    opt.plugin + "." + opt.name, tmpopt, opt.desc))
         else:
             self.ui_log.info(_("No plugin options available."))
 
@@ -911,9 +1012,7 @@ class SoSReport(SoSComponent):
                 self.ui_log.error("Exiting on user cancel")
                 self._exit(130)
             except Exception as e:
-                self.ui_log.info("")
-                self.ui_log.error(e)
-                self._exit(e)
+                self._exit(1, e)
 
     def _log_plugin_exception(self, plugin, method):
         trace = traceback.format_exc()
@@ -950,20 +1049,6 @@ class SoSReport(SoSComponent):
         self._exit(1)
 
     def setup(self):
-        # Log command line options
-        msg = "[%s:%s] executing 'sos %s'"
-        self.soslog.info(msg % (__name__, "setup", " ".join(self.cmdline)))
-
-        # Log active preset defaults
-        preset_args = self.preset.opts.to_args()
-        msg = ("[%s:%s] using '%s' preset defaults (%s)" %
-               (__name__, "setup", self.preset.name, " ".join(preset_args)))
-        self.soslog.info(msg)
-
-        # Log effective options after applying preset defaults
-        self.soslog.info("[%s:%s] effective options now: %s" %
-                         (__name__, "setup", " ".join(self.opts.to_args())))
-
         self.ui_log.info(_(" Setting up plugins ..."))
         for plugname, plug in self.loaded_plugins:
             try:
@@ -1046,16 +1131,21 @@ class SoSReport(SoSComponent):
                 _plug.manifest.add_field('end_time', end)
                 _plug.manifest.add_field('run_time', end - start)
             except TimeoutError:
-                self.ui_log.error("\n Plugin %s timed out\n" % plugin[1])
+                msg = "Plugin %s timed out" % plugin[1]
+                # log to ui_log.error to show the user, log to soslog.info
+                # so that someone investigating the sos execution has it all
+                # in one place, but without double notifying the user.
+                self.ui_log.error("\n %s\n" % msg)
+                self.soslog.info(msg)
                 self.running_plugs.remove(plugin[1])
                 self.loaded_plugins[plugin[0]-1][1].set_timeout_hit()
+                pool.shutdown(wait=True)
                 pool._threads.clear()
         if self.opts.estimate_only:
             from pathlib import Path
             tmpdir_path = Path(self.archive.get_tmp_dir())
             self.estimated_plugsizes[plugin[1]] = sum(
-                    [f.stat().st_size for f in tmpdir_path.glob('**/*')
-                     if (os.path.isfile(f) and not os.path.islink(f))])
+                    [f.lstat().st_size for f in tmpdir_path.glob('**/*')])
             # remove whole tmp_dir content - including "sos_commands" and
             # similar dirs that will be re-created on demand by next plugin
             # if needed; it is less error-prone approach than skipping
@@ -1165,13 +1255,9 @@ class SoSReport(SoSComponent):
                                         cmd['file']
                                     )))
 
-            for content, f in plug.copy_strings:
+            for content, f, tags in plug.copy_strings:
                 section.add(CreatedFile(name=f,
-                                        href=os.path.join(
-                                            "..",
-                                            "sos_strings",
-                                            plugname,
-                                            f)))
+                                        href=os.path.join("..", f)))
 
             report.add(section)
 
@@ -1281,6 +1367,34 @@ class SoSReport(SoSComponent):
                 short_name='manifest.json'
             )
 
+        # print results in estimate mode (to include also just added manifest)
+        if self.opts.estimate_only:
+            from sos.utilities import get_human_readable
+            from pathlib import Path
+            # add sos_logs, sos_reports dirs, etc., basically everything
+            # that remained in self.tmpdir after plugins' contents removal
+            # that still will be moved to the sos report final directory path
+            tmpdir_path = Path(self.tmpdir)
+            self.estimated_plugsizes['sos_logs_reports'] = sum(
+                    [f.lstat().st_size for f in tmpdir_path.glob('**/*')])
+
+            _sum = get_human_readable(sum(self.estimated_plugsizes.values()))
+            self.ui_log.info("Estimated disk space requirement for whole "
+                             "uncompressed sos report directory: %s" % _sum)
+            bigplugins = sorted(self.estimated_plugsizes.items(),
+                                key=lambda x: x[1], reverse=True)[:5]
+            bp_out = ",  ".join("%s: %s" %
+                                (p, get_human_readable(v, precision=0))
+                                for p, v in bigplugins)
+            self.ui_log.info("Five biggest plugins:  %s" % bp_out)
+            self.ui_log.info("")
+            self.ui_log.info("Please note the estimation is relevant to the "
+                             "current options.")
+            self.ui_log.info("Be aware that the real disk space requirements "
+                             "might be different. A rule of thumb is to "
+                             "reserve at least double the estimation.")
+            self.ui_log.info("")
+
         # package up and compress the results
         if not self.opts.build:
             old_umask = os.umask(0o077)
@@ -1385,24 +1499,6 @@ class SoSReport(SoSComponent):
             self.policy.display_results(archive, directory, checksum,
                                         map_file=map_file)
 
-        if self.opts.estimate_only:
-            from sos.utilities import get_human_readable
-            _sum = get_human_readable(sum(self.estimated_plugsizes.values()))
-            self.ui_log.info("Estimated disk space requirement for whole "
-                             "uncompressed sos report directory: %s" % _sum)
-            bigplugins = sorted(self.estimated_plugsizes.items(),
-                                key=lambda x: x[1], reverse=True)[:3]
-            bp_out = ",  ".join("%s: %s" %
-                                (p, get_human_readable(v, precision=0))
-                                for p, v in bigplugins)
-            self.ui_log.info("Three biggest plugins:  %s" % bp_out)
-            self.ui_log.info("")
-            self.ui_log.info("Please note the estimation is relevant to the "
-                             "current options.")
-            self.ui_log.info("Be aware that the real disk space requirements "
-                             "might be different.")
-            self.ui_log.info("")
-
         if self.opts.upload or self.opts.upload_url:
             if not self.opts.build:
                 try:
@@ -1445,11 +1541,27 @@ class SoSReport(SoSComponent):
         self.report_md.add_list('disabled_plugins', self.opts.skip_plugins)
         self.report_md.add_section('plugins')
 
+    def _merge_preset_options(self):
+        # Log command line options
+        msg = "[%s:%s] executing 'sos %s'"
+        self.soslog.info(msg % (__name__, "setup", " ".join(self.cmdline)))
+
+        # Log active preset defaults
+        preset_args = self.preset.opts.to_args()
+        msg = ("[%s:%s] using '%s' preset defaults (%s)" %
+               (__name__, "setup", self.preset.name, " ".join(preset_args)))
+        self.soslog.info(msg)
+
+        # Log effective options after applying preset defaults
+        self.soslog.info("[%s:%s] effective options now: %s" %
+                         (__name__, "setup", " ".join(self.opts.to_args())))
+
     def execute(self):
         try:
             self.policy.set_commons(self.get_commons())
             self.load_plugins()
             self._set_all_options()
+            self._merge_preset_options()
             self._set_tunables()
             self._check_for_unknown_plugins()
             self._set_plugin_options()
@@ -1490,13 +1602,15 @@ class SoSReport(SoSComponent):
         except (OSError):
             if self.opts.debug:
                 raise
-            self.cleanup()
+            if not os.getenv('SOS_TEST_LOGS', None) == 'keep':
+                self.cleanup()
         except (KeyboardInterrupt):
             self.ui_log.error("\nExiting on user cancel")
             self.cleanup()
             self._exit(130)
         except (SystemExit) as e:
-            self.cleanup()
+            if not os.getenv('SOS_TEST_LOGS', None) == 'keep':
+                self.cleanup()
             sys.exit(e.code)
 
         self._exit(1)
