@@ -8,18 +8,11 @@ from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
 import yaml
 
-from uaclient import (
-    config,
-    contract,
-    event_logger,
-    exceptions,
-    messages,
-    status,
-    util,
-)
+from uaclient import config, contract, event_logger, exceptions, messages, util
 from uaclient.defaults import DEFAULT_HELP_FILE
-from uaclient.status import (
+from uaclient.entitlements.entitlement_status import (
     ApplicabilityStatus,
+    ApplicationStatus,
     CanDisableFailure,
     CanDisableFailureReason,
     CanEnableFailure,
@@ -162,7 +155,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
     # Any custom messages to emit to the console or callables which are
     # handled at pre_enable, pre_disable, pre_install or post_enable stages
     @property
-    def messaging(self,) -> MessagingOperationsDict:
+    def messaging(self) -> MessagingOperationsDict:
         return {}
 
     def __init__(
@@ -209,7 +202,8 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 populated CanEnableFailure reason. This may expand to
                 include other types of reasons in the future.
         """
-        msg_ops = self.messaging.get("pre_enable", [])
+
+        msg_ops = self.messaging.get("pre_can_enable", [])
         if not util.handle_message_operations(msg_ops):
             return False, None
 
@@ -237,6 +231,10 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 # every other reason means we can't continue
                 return False, fail
 
+        msg_ops = self.messaging.get("pre_enable", [])
+        if not util.handle_message_operations(msg_ops):
+            return False, None
+
         ret = self._perform_enable(silent=silent)
         if not ret:
             return False, None
@@ -269,7 +267,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         """
         application_status, _ = self.application_status()
 
-        if application_status == status.ApplicationStatus.DISABLED:
+        if application_status == ApplicationStatus.DISABLED:
             return (
                 False,
                 CanDisableFailure(
@@ -314,7 +312,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             )
 
         application_status, _ = self.application_status()
-        if application_status != status.ApplicationStatus.DISABLED:
+        if application_status != ApplicationStatus.DISABLED:
             return (
                 False,
                 CanEnableFailure(
@@ -327,7 +325,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             return (False, CanEnableFailure(CanEnableFailureReason.IS_BETA))
 
         applicability_status, details = self.applicability_status()
-        if applicability_status == status.ApplicabilityStatus.INAPPLICABLE:
+        if applicability_status == ApplicabilityStatus.INAPPLICABLE:
             return (
                 False,
                 CanEnableFailure(
@@ -363,11 +361,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for service in services:
             try:
-                ent_cls = entitlement_factory(service)
+                ent_cls = entitlement_factory(cfg=self.cfg, name=service)
             except EntitlementNotFoundError:
                 continue
             ent_status, _ = ent_cls(self.cfg).application_status()
-            if ent_status == status.ApplicationStatus.ENABLED:
+            if ent_status == ApplicationStatus.ENABLED:
                 return True
 
         return False
@@ -396,9 +394,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for required_service in self.required_services:
             try:
-                ent_cls = entitlement_factory(required_service)
+                ent_cls = entitlement_factory(
+                    cfg=self.cfg, name=required_service
+                )
                 ent_status, _ = ent_cls(self.cfg).application_status()
-                if ent_status != status.ApplicationStatus.ENABLED:
+                if ent_status != ApplicationStatus.ENABLED:
                     return False
             except exceptions.EntitlementNotFoundError:
                 pass
@@ -412,7 +412,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         ret = []
         for service in self.incompatible_services:
             ent_status, _ = service.entitlement(self.cfg).application_status()
-            if ent_status == status.ApplicationStatus.ENABLED:
+            if ent_status == ApplicationStatus.ENABLED:
                 ret.append(service)
 
         return ret
@@ -428,7 +428,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return len(self.blocking_incompatible_services()) > 0
 
     def handle_incompatible_services(
-        self
+        self,
     ) -> Tuple[bool, Optional[messages.NamedMessage]]:
         """
         Prompt user when incompatible services are found during enable.
@@ -449,7 +449,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             path_to_value="features.block_disable_on_enable",
         )
         for service in self.blocking_incompatible_services():
-            ent = service.entitlement(self.cfg)
+            ent = service.entitlement(self.cfg, assume_yes=True)
 
             user_msg = messages.INCOMPATIBLE_SERVICE.format(
                 service_being_enabled=self.title,
@@ -474,14 +474,14 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             )
             event.info(disable_msg)
 
-            ret = ent.disable()
+            ret = ent.disable(silent=True)
             if not ret:
                 return ret, None
 
         return True, None
 
     def _enable_required_services(
-        self
+        self,
     ) -> Tuple[bool, Optional[messages.NamedMessage]]:
         """
         Prompt user when required services are found during enable.
@@ -494,7 +494,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for required_service in self.required_services:
             try:
-                ent_cls = entitlement_factory(required_service)
+                ent_cls = entitlement_factory(
+                    cfg=self.cfg, name=required_service
+                )
             except exceptions.EntitlementNotFoundError:
                 msg = messages.REQUIRED_SERVICE_NOT_FOUND.format(
                     service=required_service
@@ -504,8 +506,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             ent = ent_cls(self.cfg, allow_beta=True)
 
             is_service_disabled = (
-                ent.application_status()[0]
-                == status.ApplicationStatus.DISABLED
+                ent.application_status()[0] == ApplicationStatus.DISABLED
             )
 
             if is_service_disabled:
@@ -539,7 +540,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return True, None
 
     def applicability_status(
-        self
+        self,
     ) -> Tuple[ApplicabilityStatus, Optional[messages.NamedMessage]]:
         """Check all contract affordances to vet current platform
 
@@ -563,8 +564,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 return ApplicabilityStatus.INAPPLICABLE, error_message
         affordances = entitlement_cfg["entitlement"].get("affordances", {})
         platform = util.get_platform_info()
-        affordance_arches = affordances.get("architectures", [])
-        if affordance_arches and platform["arch"] not in affordance_arches:
+        affordance_arches = affordances.get("architectures", None)
+        if (
+            affordance_arches is not None
+            and platform["arch"] not in affordance_arches
+        ):
             return (
                 ApplicabilityStatus.INAPPLICABLE,
                 messages.INAPPLICABLE_ARCH.format(
@@ -573,8 +577,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                     supported_arches=", ".join(affordance_arches),
                 ),
             )
-        affordance_series = affordances.get("series", [])
-        if affordance_series and platform["series"] not in affordance_series:
+        affordance_series = affordances.get("series", None)
+        if (
+            affordance_series is not None
+            and platform["series"] not in affordance_series
+        ):
             return (
                 ApplicabilityStatus.INAPPLICABLE,
                 messages.INAPPLICABLE_SERIES.format(
@@ -582,10 +589,10 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 ),
             )
         kernel = platform["kernel"]
-        affordance_kernels = affordances.get("kernelFlavors", [])
+        affordance_kernels = affordances.get("kernelFlavors", None)
         affordance_min_kernel = affordances.get("minKernelVersion")
         match = re.match(RE_KERNEL_UNAME, kernel)
-        if affordance_kernels:
+        if affordance_kernels is not None:
             if not match or match.group("flavor") not in affordance_kernels:
                 return (
                     ApplicabilityStatus.INAPPLICABLE,
@@ -656,7 +663,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for dependent_service in self.dependent_services:
             try:
-                ent_cls = entitlement_factory(dependent_service)
+                ent_cls = entitlement_factory(
+                    cfg=self.cfg, name=dependent_service
+                )
             except exceptions.EntitlementNotFoundError:
                 msg = messages.DEPENDENT_SERVICE_NOT_FOUND.format(
                     service=dependent_service
@@ -664,10 +673,10 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 event.info(info_msg=msg.msg, file_type=sys.stderr)
                 return False, msg
 
-            ent = ent_cls(self.cfg)
+            ent = ent_cls(cfg=self.cfg, assume_yes=True)
 
             is_service_enabled = (
-                ent.application_status()[0] == status.ApplicationStatus.ENABLED
+                ent.application_status()[0] == ApplicationStatus.ENABLED
             )
 
             if is_service_enabled:
@@ -710,13 +719,16 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         """Check if system needs to be rebooted."""
         return util.should_reboot()
 
-    def _check_for_reboot_msg(self, operation: str) -> None:
+    def _check_for_reboot_msg(
+        self, operation: str, silent: bool = False
+    ) -> None:
         """Check if user should be alerted that a reboot must be performed.
 
         @param operation: The operation being executed.
+        @param silent: Boolean set True to silence print/log of messages
         """
-        if self._check_for_reboot():
-            print(
+        if self._check_for_reboot() and not silent:
+            event.info(
                 messages.ENABLE_REBOOT_REQUIRED_TMPL.format(
                     operation=operation
                 )
@@ -763,7 +775,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         if not util.handle_message_operations(msg_ops):
             return False, None
 
-        self._check_for_reboot_msg(operation="disable operation")
+        self._check_for_reboot_msg(
+            operation="disable operation", silent=silent
+        )
         return True, None
 
     def contract_status(self) -> ContractStatus:
@@ -787,12 +801,12 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             return False
         return True
 
-    def _check_application_status_on_cache(self) -> status.ApplicationStatus:
+    def _check_application_status_on_cache(self) -> ApplicationStatus:
         """Check on the state of application on the status cache."""
         status_cache = self.cfg.read_cache("status-cache")
 
         if status_cache is None:
-            return status.ApplicationStatus.DISABLED
+            return ApplicationStatus.DISABLED
 
         services_status_list = status_cache.get("services", [])
 
@@ -801,11 +815,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 service_status = service.get("status")
 
                 if service_status == "enabled":
-                    return status.ApplicationStatus.ENABLED
+                    return ApplicationStatus.ENABLED
                 else:
-                    return status.ApplicationStatus.DISABLED
+                    return ApplicationStatus.DISABLED
 
-        return status.ApplicationStatus.DISABLED
+        return ApplicationStatus.DISABLED
 
     def process_contract_deltas(
         self,
@@ -836,7 +850,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         transition_to_unentitled = bool(delta_entitlement == util.DROPPED_KEY)
         if not transition_to_unentitled:
             if delta_entitlement:
-                util.apply_series_overrides(deltas)
+                util.apply_contract_overrides(deltas)
                 delta_entitlement = deltas["entitlement"]
             if orig_access and "entitled" in delta_entitlement:
                 transition_to_unentitled = delta_entitlement["entitled"] in (
@@ -849,7 +863,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             else:
                 application_status, _ = self.application_status()
 
-            if application_status != status.ApplicationStatus.DISABLED:
+            if application_status != ApplicationStatus.DISABLED:
                 if self.can_disable():
                     self.disable()
                     logging.info(
@@ -897,7 +911,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return False
 
     def user_facing_status(
-        self
+        self,
     ) -> Tuple[UserFacingStatus, Optional[messages.NamedMessage]]:
         """Return (user-facing status, details) for entitlement"""
         applicability, details = self.applicability_status()
@@ -917,15 +931,15 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         application_status, explanation = self.application_status()
         user_facing_status = {
-            status.ApplicationStatus.ENABLED: UserFacingStatus.ACTIVE,
-            status.ApplicationStatus.DISABLED: UserFacingStatus.INACTIVE,
+            ApplicationStatus.ENABLED: UserFacingStatus.ACTIVE,
+            ApplicationStatus.DISABLED: UserFacingStatus.INACTIVE,
         }[application_status]
         return user_facing_status, explanation
 
     @abc.abstractmethod
     def application_status(
-        self
-    ) -> Tuple[status.ApplicationStatus, Optional[messages.NamedMessage]]:
+        self,
+    ) -> Tuple[ApplicationStatus, Optional[messages.NamedMessage]]:
         """
         The current status of application of this entitlement
 
